# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
from urllib import quote as url_quote

# zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from OFS.ObjectManager import BadRequestException
from OFS.ObjectManager import checkValidId
from OFS.ObjectManager import REPLACEABLE
from AccessControl.Permissions import view, view_management_screens

# foreign product imports
from Products.OrderedFolder.OrderedFolder import OrderedFolder
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

# product imports
import Components
from Components import manage_addFieldRowComponent
from Renderable import Renderable
from Utility import getTemplateFile
from Const import *

containerClasses = []

manage_addPageForm = getTemplateFile('manage_addPageForm', globals())
def manage_addPage(self, id, title='', css_class='', width='', use_caption=0,
                   alignment=ALIGNMENT_LINEAR, REQUEST=None):
    '''FIXME: I need a docstring'''

    page  = Page(id, title=title, css_class=css_class, width=width, use_caption=use_caption, alignment=alignment)
    self._setObject(page.id, page)
    ob = self._getOb(page.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob
    
manage_addPanelForm = getTemplateFile('manage_addPanelForm', globals())
def manage_addPanel(self, id, title='', css_class='', width='', use_caption=0,
                    alignment=ALIGNMENT_LINEAR, REQUEST=None):
    '''FIXME: I need a docstring'''

    panel  = Panel(id, title=title, css_class=css_class, width=width,
                   use_caption=use_caption, alignment=alignment)
    self._setObject(panel.id, panel)
    ob = self._getOb(panel.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

manage_addFieldListForm = getTemplateFile('manage_addFieldListForm', globals())
def manage_addFieldList(self, id, title='', css_class='', width='', use_caption=0, field_ids=[],
                        alignment=ALIGNMENT_VERTICAL, create_default_columns=0, REQUEST=None):
    '''FIXME: I need a docstring'''

    field_list = FieldList(id, title=title, css_class=css_class, width=width,
                           use_caption=use_caption, alignment=alignment,
                           create_default_columns=create_default_columns)
    self._setObject(field_list.id, field_list)
    ob = self._getOb(field_list.id).__of__(self)
    manage_addFieldRowComponent(ob, field_ids)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob
    
class ComponentManager (OrderedFolder):
    """ Mixin Class for AbstractContainer and FormulatorLayout
        provides a single method <getComponents>, that returns a filtered list
        of components
    """
    security = ClassSecurityInfo()
    manage_options = ()
    for option in OrderedFolder.manage_options:
        if option['label'] == 'View':
            option['label'] = 'Test'
        manage_options += (option,)
    


    security.declareProtected(view, 'getComponents')
    def getComponents(self, klass=None):
        ''' get all components in this component manager '''
        c = self.objectValues()
        klass = klass or Renderable
        c = filter(lambda inst, klass=klass: issubclass(inst.__class__, klass), c)
        return c

    security.declareProtected(view, 'getValidComponents')
    def getValidComponents(self, template_type, klass=None):
        ''' get all components in this component manager '''
        REQUEST = self.REQUEST
        c = self.getComponents(klass)
        c = filter(lambda x, t=template_type, req=REQUEST: x.isValidFor(t), c)
        return c

    def getAllComponents(self, klass=None):
        ''' recursively get all components valid for template_type and REQUEST '''
        c = self.getComponents(klass)
        for component in self.getComponents(ComponentManager):
            c.extend(component.getAllComponents(klass))
        return c
            
    def getAllValidComponents(self, template_type):
        ''' recursively get all components valid for template_type and REQUEST '''
        REQUEST = self.REQUEST
        c = self.getValidComponents(template_type)
        for component in filter(lambda inst, klass=ComponentManager: issubclass(inst.__class__, klass), c):
            c.extend(component.getAllValidComponents(template_type))
        return c
            
    security.declareProtected(view, 'getHiddenFields')
    def getHiddenFields(self):
        ''' return a list of fields that are valid for the current template type but hidden for REQUEST '''
        c = self.getAllComponents(klass=Components.FieldComponent)
        fields = map(lambda x: x.getField(), c)
        return filter(lambda x:x.get_value('hidden'), fields)

    security.declarePrivate('getBrokenFieldComponents')
    def getBrokenFieldComponents(self):
        ''' return a list of all broken field components '''
        components = self.getAllComponents(klass=Components.FieldComponent)
        components = filter(lambda x: x.isBroken(), components)
        return components
    
    security.declarePrivate('getFieldIds')
    def getFieldIds(self):
        ids = []
        for ob in self.objectValues():
            if hasattr(ob.aq_base, 'getFieldIds'):
                ids.extend(ob.aq_base.getFieldIds())
        return ids

        
    

InitializeClass(ComponentManager)

class AbstractContainer(Renderable, ComponentManager):
    security = ClassSecurityInfo()
    selected_meta_types = ()
    
    _properties = OrderedFolder._properties + \
                  ({'id': 'alignment', 'type': 'selection', 'mode':'w', 'default': ALIGNMENT_LINEAR,
                    'select_variable': 'getAlignments'},
                   {'id': 'width', 'type': 'string', 'mode':'w', 'default':''},
                   {'id': 'css_class', 'type': 'string', 'mode':'w', 'default': ''},
                   {'id': 'use_caption', 'type': 'boolean', 'mode':'w', 'default': '0'}
                   )

    def __init__(self, id, title='', css_class='', width='', use_caption=0, alignment=ALIGNMENT_LINEAR):
        self.id = id
        self.title = title
        self.css_class = css_class
        self.width = width
        self.use_caption = use_caption
        self.alignment = alignment
        for property in self._properties:
            if not hasattr(self, property['id']):
                setattr(self, property['id'], property['default'])

    security.declarePrivate('getBoundNames')
    def getBoundNames(self, template_type=TEMPLATE_TYPE_FORM, **kw):
        bound_names = AbstractContainer.inheritedAttribute('getBoundNames') \
                      (self, template_type=template_type, **kw)
        form = self.getForm()
        components = self.getValidComponents(template_type)
        bound_names.update({'form': form, 'components': components})
        return bound_names

    _form_template = None
    _report_template = None

            
InitializeClass(AbstractContainer)

class Page(AbstractContainer):
    """ Page(AbstractContainer) """
    
    meta_type = "Form Page"
    selected_meta_types = ('Panel', 'Field List', 'Field Component', 'Label Component',
                           'Button Component', 'Error Component')
    security = ClassSecurityInfo()
    icon = 'misc_/FormulatorLayout/Page'
    _properties = AbstractContainer._properties + \
                  ({'id': 'use_default_button', 'type': 'boolean', 'mode': 'w', 'default': 1},)

    _form_template = getTemplateFile('templates/Page')
    _report_template = getTemplateFile('templates/Page_report')
    
InitializeClass(Page)
containerClasses.append(Page)

class Panel(AbstractContainer):
    """ Panel(AbstractContainer) """
    meta_type = "Panel"
    selected_meta_types = ('Panel', 'Field List', 'Field Component', 'Label Component',
                           'Button Component', 'Error Component')
    security = ClassSecurityInfo()
    icon = 'misc_/FormulatorLayout/Panel'
    
    _form_template = getTemplateFile('templates/Panel')
    _report_template = getTemplateFile('templates/Panel_report')
    
InitializeClass(Panel)
containerClasses.append(Panel)

class FieldList(Panel):
    """ FieldList(Panel) """
    meta_type = 'Field List'
    selected_meta_types = ('Field Row Component',)
    security = ClassSecurityInfo()
    icon = 'misc_/FormulatorLayout/FieldList'
    
    def __init__(self, id, title='', css_class='', width='', use_caption=0, alignment=ALIGNMENT_VERTICAL,
                 create_default_columns=0):
        AbstractContainer.__init__(self, id, title=title, css_class=css_class, width=width,
                                   use_caption=use_caption, alignment=alignment)
        columns = ColumnList('columns', title='Column Definitions', create_default_columns=create_default_columns)
        self._setObject('columns', columns)
        columns = self._getOb('columns')
    
    security.declareProtected(view, 'getSelectedColumns')
    def getValidColumns(self, template_type=TEMPLATE_TYPE_DEFAULT, REQUEST=None):
        request = REQUEST or self.REQUEST
        columns = self.columns.getValidComponents(template_type=template_type)
        return columns

    _form_template = getTemplateFile('templates/FieldList')
    _report_template = getTemplateFile('templates/FieldList_report')

InitializeClass(FieldList)
containerClasses.append(FieldList)

class ColumnList(ComponentManager):
    """ """
    meta_type = "Column List"
    selected_meta_types = ('Field Column Component', 'Error Column Component')
    security = ClassSecurityInfo()
    icon = 'misc_/FormulatorLayout/ColumnList'
    _properties = ComponentManager._properties + \
                  ({'id': 'alignment', 'type': 'selection', 'mode':'w', 'default': ALIGNMENT_HORIZONTAL,
                    'select_variable': 'getAlignments'},)


    def __init__(self, id, title='', alignment=ALIGNMENT_HORIZONTAL, create_default_columns=0):
        self.id = str(id)
        self.title = title
        self.alignment = alignment
        if create_default_columns:
            createDefaultColumns(self)
    
InitializeClass(ColumnList)
containerClasses.append(ColumnList)

    
def createDefaultColumns(column_list):
    from Components import FieldColumnComponent, ErrorColumnComponent
    columns = []
    columns.append(FieldColumnComponent('field_title', text="""<span tal:attributes="class template/css_class"
    tal:content="python:field.get_value('title')">FIELD_TITLE</span>""",
                                        css_class="fld-title", width="100px"))
    columns.append(FieldColumnComponent('field_render', text="""<span tal:attributes="class template/css_class"
    tal:content="structure python:field.render(REQUEST=request, value=field_value)">FIELD_RENDER</span>""",
                                        css_class="fld-elem", valid_for=TEMPLATE_TYPE_FORM))
    columns.append(FieldColumnComponent('field_value', text="""<span tal:attributes="class template/css_class"
    tal:content="structure field_value">FIELD_VALUE</span>""",
                                        css_class="fld-elem", valid_for=TEMPLATE_TYPE_REPORT))
    columns.append(FieldColumnComponent('field_description', text="""<span
    tal:content="structure python:field.get_value('description')">DESCRIPTION</span>""",
                                        css_class="fld-help", width=""))
    columns.append(ErrorColumnComponent('error', css_class="fld-error", text="""<div
    tal:define="error python:template.getErrorForField(field.id, errors)"
    tal:condition="error"
    tal:attributes="class template/getCSSClass">
    <span tal:replace="error/error_text">ERROR</span>
    </div>
"""))
    for column in columns:
        column_list._setObject(column.id, column)
