# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
from urllib import quote as url_quote, unquote

# zope imports
from AccessControl import SecurityInfo
from AccessControl.Permissions import import_export_objects
from Globals import InitializeClass, DTMLFile
from OFS.Folder import Folder
from OFS.ObjectManager import checkValidId
from Products.Formulator import Form
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport

# product imports
import FormulatorImportExport
import FormulatorLayout
from Utility import getTemplateFile

manage_addFormFolderForm = getTemplateFile('manage_addFormFolderForm', globals())
def manage_addFormFolder(self, id, title='', REQUEST=None):
    '''manage_addFormFolder(self, id, title='', REQUEST=None) -->
       add FormFolder;
       return None or HTML'''

    ob  = FormFolder(id, title=title)
    self._setObject(ob.id, ob)
    ob = self._getOb(ob.id).__of__(self)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')
    else:
        return ob

class FormFolder(SubObjectTypeSupport, Folder):
    """ Container Class for Formulator Forms. """
    meta_type = "Form Folder"
    icon = 'misc_/FormulatorLayout/FormFolder'
    selected_meta_types = (Form.ZMIForm.meta_type, FormulatorLayout.FormulatorLayout.meta_type)
    manage_options = Folder.manage_options + ({'label':'Import / Export Form', 'action':'manage_form_import_export_form'},)
    security = SecurityInfo.ClassSecurityInfo()

    security.declareProtected(import_export_objects, 'manage_form_import_form')    
    manage_form_import_export_form = DTMLFile('templates/FormFolder_manage_form_import_export_form', globals())
    # Uncomment the next line to allow selection of allowed objects for this container
    # manage_options = manage_options + SubObjectTypeSupport.manage_options

    def __init__(self, id, title=''):
        self.id = id
        self.title = title
    security.declareProtected(import_export_objects, 'manage_form_export')
    def manage_form_export(self, form_id, REQUEST=None, RESPONSE=None):
        """ export a single formulator form """
        if not REQUEST:
            REQUEST = self.REQUEST
        if not RESPONSE:
            RESPONSE = REQUEST.RESPONSE

        form = getattr(self, form_id, None)
        if form:
            RESPONSE.setHeader("content-type", "text/xml")
            RESPONSE.setHeader("content-disposition", "filename=%s.xml" % url_quote(form.getId()))
            frm_ex = FormulatorImportExport.FormulatorExporter()
            return frm_ex.exportXML(form)

    security.declareProtected(import_export_objects, 'manage_form_import')        
    def manage_form_import(self, file_or_filename, REQUEST=None, RESPONSE=None):
        """ import a form a file """
        frm_imp = FormulatorImportExport.FormulatorImporter()
        form = frm_imp.importXML(file_or_filename)
        id = str(form.getId())
        checkValidId(self, id)
        self._setObject(id, form)
        if REQUEST:
            return self.manage_main(self, REQUEST)
            
InitializeClass(FormFolder)
