from types import StringType
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view
from Const import FL_OPTIONS_KEY, TEMPLATE_TYPE_FORM, TEMPLATE_TYPE_REPORT, TEMPLATE_TYPE_DEFAULT

class Renderable:
    """ Renderable: mixin class to support template handling for formulator-layout containers and components """
    security = ClassSecurityInfo()
    
    security.declareProtected(view, 'getTemplate')
    def getTemplate(self, template_type=TEMPLATE_TYPE_DEFAULT):
        """ """
        # try to get a customized local template
        t = getattr(self.aq_base, 'render_%s.pt' % template_type, None)
        if t is not None:
            return t
        # try to get a customized global (acquired) template
        t = getattr(self, 'render%s_%s.pt' % (self.__class__.__name__, template_type), None)
        if t is not None:
            return t
        # return default template
        return getattr(self, '_%s_template' % template_type)
    
    security.declareProtected(view, 'render')
    def render(self, template_type=TEMPLATE_TYPE_DEFAULT, **kw):
        bound_names = self.getBoundNames(template_type=template_type, **kw)
        template = self.getTemplate(template_type)
        if template is None:
            return ''
        return template._exec(bound_names, (), kw)
    
    security.declareProtected(view, 'renderReport')
    def renderReport(self, **kw):
        return self.render(template_type=TEMPLATE_TYPE_REPORT, **kw)
    
    security.declarePrivate('getBoundNames')
    def getBoundNames(self, template_type=TEMPLATE_TYPE_DEFAULT, **kw):
        """ return a dictionary with additional names to provide to associated templates
            default: ob->object currently rendered (or None if no object is rendered)
        """
        ob = self.getRequestOption('ob')
        return {'ob': ob}

    security.declarePrivate('isValidFor')
    def isValidFor(self, template_type):
        valid_for = self.getProperty('valid_for', None)
        if StringType is type(valid_for):
            valid_for = (valid_for,)
        if valid_for is not None and not template_type in valid_for:
            return 0
        return 1
    
    security.declareProtected(view, 'getCSSClass')
    def getCSSClass(self):
        ''' return a css_class property or our classname as default '''
        css_class = getattr(self, 'css_class', None)
        return css_class or self.__class__.__name__
        
    security.declareProtected(view, 'index_html')
    def index_html(self, REQUEST):
        """ test this component """
        return self.render()

InitializeClass(Renderable)
